/*
 Mattias P. Heinrich
 Universitaet Luebeck, 2014
 */
template <typename Type>

void readNifti(string filestr,Type*& vol,char*& header,int& m,int& n,int& o){
    //convert input string into char* array
    char* filename=new char[filestr.size()+1];
    copy(filestr.begin(),filestr.end(),filename);
    filename[filestr.size()]='\0';
    ifstream file(filename,ios::in|ios::binary);
    //opens file for binary-input
    if(file.is_open()){
        //read header with image dimensions
        file.read(header,352);
        short* dimensions;
        dimensions=reinterpret_cast<short*>(header+40);
        m=(int)dimensions[1];
        n=(int)dimensions[2];
        o=(int)dimensions[3];
        //read float array
        vol=new Type[m*n*o];
        file.read((char*)vol,m*n*o*sizeof(Type));
        //binary are read character by character
        //reinterpre_cast converts them to float/short
        vol=reinterpret_cast<Type*>(vol);
        //printf("Read image with dimensions %dx%dx%d\n",m,n,o);
    }
    else{
        printf("File error. Did not find file. Exiting.\n");
        exit(1);
    }
    
    file.close();
    
}


void writeNifti(string filestr,float* vol,char* header,int m,int n,int o){
    //convert input string into char* array
    char* filename=new char[filestr.size()+1];
    copy(filestr.begin(),filestr.end(),filename);
    filename[filestr.size()]='\0';
    
    char* header2=new char[352];
    copy(header,header+352,header2);
    int dim=o>1?3:2;
    short* dimensions=new short[4];
    dimensions[0]=dim; dimensions[1]=m; dimensions[2]=n; dimensions[3]=o;
    char* dimchar=reinterpret_cast<char*>(dimensions);
    copy(dimchar,dimchar+8,header2+40);
    
    short* datatype=new short[2];
    datatype[0]=16; datatype[1]=32; //float datatype
    char* datachar=reinterpret_cast<char*>(datatype);
    copy(datachar,datachar+4,header2+70);
    
    printf("Writing image with dimensions %dx%dx%d\n",m,n,o);
    
    ofstream file(filename,ios::out|ios::binary);
    //opens file for binary-output
	if(file.is_open()){
		file.write(header2,352);
		file.write(reinterpret_cast<char*>(vol),m*n*o*4);
		file.close();
		cout<<"File "<<filename<<" written.\n";
	}
    else{
        printf("File error. Could not write file.\n");
    }
}

void writeSegment(string filestr,short* vol,char* header,int m,int n,int o){
    //convert input string into char* array
    char* filename=new char[filestr.size()+1];
    copy(filestr.begin(),filestr.end(),filename);
    filename[filestr.size()]='\0';
    
    char* header2=new char[352];
    copy(header,header+352,header2);
    int dim=o>1?3:2;
    short* dimensions=new short[4];
    dimensions[0]=dim; dimensions[1]=m; dimensions[2]=n; dimensions[3]=o;
    char* dimchar=reinterpret_cast<char*>(dimensions);
    copy(dimchar,dimchar+8,header2+40);
    
    short* datatype=new short[2];
    datatype[0]=4; datatype[1]=512; //short datatype
    char* datachar=reinterpret_cast<char*>(datatype);
    copy(datachar,datachar+4,header2+70);
    
    printf("Writing segmentation with dimensions %dx%dx%d\n",m,n,o);
    
    ofstream file(filename,ios::out|ios::binary);
    //opens file for binary-output
	if(file.is_open()){
		file.write(header2,352);
		file.write(reinterpret_cast<char*>(vol),m*n*o*2);
		file.close();
		cout<<"File "<<filename<<" written.\n";
	}
    else{
        printf("File error. Could not write file.\n");
    }
}


